@@ -1,4 +1,4 @@
-use crate::{sys, utils};
+use crate::Backend;
 use core::ptr;
 
 #[cfg(target_pointer_width = "64")]
@@ -14,29 +14,30 @@ pub enum ReserveType {
 
 pub struct ReserveHeader {
     offset: u32,
+    size: usize,
     pub ty: ReserveType,
 }
 
-pub fn new(size: usize, ty: ReserveType) -> (usize, *mut ReserveHeader) {
+pub fn new<B: Backend>(size: usize, ty: ReserveType) -> (usize, *mut ReserveHeader) {
     if size >= usize::MAX - RESERVE_ALIGN {
         return (0, ptr::null_mut());
     }
 
     let total_size = RESERVE_ALIGN + size;
-    let base = unsafe { sys::__haz_alloc_mreserve(ptr::null_mut(), total_size) };
+    let base = B::mreserve(ptr::null_mut(), total_size);
     if base.is_null() {
         return (0, ptr::null_mut());
     }
 
     let offset = base.align_offset(RESERVE_ALIGN);
     let ptr = unsafe { base.add(offset) as *mut ReserveHeader };
-    if unsafe { !sys::__haz_alloc_mcommit(ptr as *mut u8, utils::page_size()) } {
+    if unsafe { !B::mcommit(ptr as *mut u8, B::pagesize()) } {
         return (0, ptr::null_mut());
     }
-    
     unsafe {
         ptr.write(ReserveHeader {
             offset: offset as u32,
+            size: total_size,
             ty,
         });
     }
@@ -45,7 +46,7 @@ pub fn new(size: usize, ty: ReserveType) -> (usize, *mut ReserveHeader) {
 }
 
 #[inline]
-pub unsafe fn delete(ptr: *mut ReserveHeader) {
+pub unsafe fn delete(munreserve: unsafe fn(ptr: *mut u8, size: usize), ptr: *mut ReserveHeader) {
     let offset = (*ptr).offset;
-    sys::__haz_alloc_munreserve((ptr as *mut u8).sub(offset as usize));
+    munreserve((ptr as *mut u8).sub(offset as usize), (*ptr).size);
 }
